// Bruch.java                                   MM 2007

import Tools.Euklid.*;                       // Kuerzen

/**
 * Gemeine Brueche.
 */
public class Bruch
{
                                    // Instanzvariablen
/**
 * Zaehler.
 */
  private int zaehler;

/**
 * Nenner.
 */
  private int nenner;

/* ------------------------------------------------- */
                                       // Konstruktoren
/**
 * Konstruktor,
 * wirft Ausnahme, falls Nenner == 0.
 * @param z Zaehler
 * @param n Nenner
 */
  public Bruch( int z, int n)
  throws Exception    // BruchException, NPlusException
  {
    if( n == 0) throw new BruchException();
    if( z == 0) n = 1;

// Kuerzen
    int i = Euklid.ggT( Math.abs( z), Math.abs( n));
    if( i > 1)
    {
      zaehler = z / i;
      nenner = n / i;
    }
    else
    {
      zaehler = z;
      nenner = n;
    }
  }

/**
 * Konstruktor, ganze Zahl als Bruch.
 * @param z ganze Zahl
 */
  public Bruch( int z)
  {
      zaehler = z;
      nenner = 1;
  }

/* ------------------------------------------------- */
                                        // get-Methoden
/**
 * Lesen des Zaehlers.
 */
  public int getZaehler()
  {
    return zaehler;
  }

/**
 * Lesen des Nenners.
 */
  public int getNenner()
  {
    return nenner;
  }

/* ------------------------------------------------- */
                                    // service-Methoden
/**
 * Setzen des Zaehlers.
 */
  public void setZaehler( int z)
  {
    zaehler = z;
  }

/**
 * Setzen des Nenners.
 */
  public void setNenner( int n)
  throws Exception    // BruchException, NPlusException
  {
    if( n == 0) throw new BruchException();
    nenner = n;
  }

/* ------------------------------------------------- */
                                         // Operationen
/**
 * Addition.
 */
  public Bruch add( Bruch a)
  throws Exception    // BruchException, NPlusException
  {
    int x = zaehler  * a.getNenner()
          + nenner * a.getZaehler();
    int y = nenner * a.getNenner();
    return new Bruch( x, y);
  }

/* ------------------------------------------------- */
                                    // toString-Methode
/**
 * Darstellung eines Bruchs.
 */
  public String toString()
  {
     return "" + zaehler + " / " + nenner;
  }

/* ------------------------------------------------- */
/* ------------------------------------------------- */
/**
 * Testprogramm zur Klasse Bruch,
 * 1: 2 / 3 + 19 / 39 = 15 / 13.
 * 2: 2 + 2 + 2 - 1 = 5 / 1
 */
  public static void main( String[] args)
  {
    try
    {
      Bruch a = new Bruch( 2, 3);
      Bruch b = new Bruch( 19, 39);

      System.out.println
      ( "1: " + a + " + " + b + " = " + ( a.add( b)));

      Bruch c = new Bruch( 2);
      Bruch d = new Bruch( -1);

      System.out.println
      ( "2: 3 * " + c + " + " + d + " = "
        + d.add( c.add( c.add( c))));
    }
    catch( Exception e)
    {
      System.out.println( "Fehler: " + e);
    }
  }
}
