//ChatClientProtokoll.java                      MM 2011

import java.io.*;                          // Exception
import java.net.*;                            // Socket

/**
 * Protokoll eines Clients,
 * zeigt Nachrichten vom Server an
 * und schickt Nachrichten zum Server.
 */
class ChatClientProtokoll
  extends Thread
{
/**
 * Socket fuer Serververbindung
 */
  private Socket c;

/**
 * Eingabestrom vom Server
 */
  private BufferedReader vomServer;

/**
 * Ausgabestrom zum Server
 */
  private PrintWriter zumServer;

/**
 * Konstruktor,
 * baut Datenstroeme zum/vom Server auf.
 * @param c Socket fuer Serververbindung
 */
  public ChatClientProtokoll( Socket c)
  {
    try
    {
// Socket, Datenstroeme
      this.c = c;

      zumServer =  new PrintWriter
        ( c.getOutputStream(), true);

      vomServer = new BufferedReader
        ( new InputStreamReader( c.getInputStream()));

// Thread
      start();
    }
    catch( Exception e)
    {
      System.out.println
      ( "\nProtokoll konnte nicht gestartet werden\n  "
        + e);
    }
  }

/**
 * Interaktion,
 * liest und verarbeitet Nachrichten vom Server.
 */
  public void run()
  {
    try
    {
      String nachricht;

      while( true)
      {
        nachricht = vomServer.readLine();
        if( nachricht != null)
          System.out.println( nachricht);
      }
    }
    catch( Exception e)
    {
      if( !isInterrupted())
      {
        System.out.println
        ( "\nVerbindung zum Server unterbrochen\n  " + e);
        release();
      }
    }
  }

/**
 * Nutzereingaben,
 * Eingabe und Versenden von Nachrichten.
 */
  public void action()
  {
    BufferedReader vonTastatur = new BufferedReader
      ( new InputStreamReader( System.in));

    System.out.println
      ( "Dialog gestartet (Dialog mit QUIT beenden)");
    try
    {
      String nachricht;

      do
      {
        nachricht = vonTastatur.readLine();

        if( nachricht.equalsIgnoreCase( "quit"))
        {
          release();
          return;
        }
        else zumServer.println( nachricht);

      } while( true);
    }
    catch( Exception e)
    {
      System.out.println
      ( "\nVerbindung zum Server unterbrochen\n  " + e);
      release();
    }
  }

/**
 * Beendet ChatClient,
 * meldet Abbruch dem Server,
 * bricht Verbindung zum Server ab,
 * bricht Programm ab.
 */
  protected void release()
  {
// Thread beenden
    interrupt();

    try
    {
// Server informieren
      System.out.println( "Dialog beendet");
      zumServer.println( "quit");

// Clientsocket schliessen
      c.close();
      System.out.println( "Client abgemeldet");
    }
    catch( Exception e)
    {
      System.out.println
      ( "\nFehler beim Schliessen des Client\n  " + e);
    }

// Programm beenden
    System.exit( 0);
  }
}