//ChatServerProtokoll.java                      MM 2005

import java.util.*;                         // Observer
import java.io.*;                          // Exception
import java.net.*;                            // Socket

/**
 * Steuert Interaktion zwischen Server und Client,
 * ueberwacht ChatModel (Original).
 */
class ChatServerProtokoll
  extends Thread
  implements Observer
{
/**
 * Model, Original serverseitig.
 */
  private ChatModel model;

/**
 * Clientzaehler
 */
  private static int anzahl = 0;

/**
 * Nummer des Client
 */
  private int nr;

/**
 * Socket fuer Clientverbindung
 */
  private Socket s;

/**
 * Eingabestrom vom Client
 */
  private BufferedReader vomClient;

/**
 * Ausgabestrom zum Client
 */
  private PrintWriter zumClient;

/**
 * Konstruktor,
 * baut Ueberwachungsmechanismus
 * und Datenstroeme zum/vom Client auf.
 * @param s Socket fuer Clientverbindung
 * @param m Modell auf dem Server
 */
  public ChatServerProtokoll( Socket s, ChatModel m)
  {
// Socket
    this.s = s;

// Modell, Ueberwachungsmechanismus
    model = m;
    model.addObserver( this);

    nr = ++anzahl;
    try
    {
// Datenstroeme
      vomClient = new BufferedReader
        ( new InputStreamReader( s.getInputStream()));

      zumClient = new PrintWriter
        ( s.getOutputStream(), true);

// Anfangszustand
      model.setNachricht
      ( "\tClient " + nr + " betritt den Chatroom," +
        " Personen im Chatroom: " +
        model.countObservers());

// Thread
      start();
    }
    catch( Exception e)
    {
      System.out.println
        ( "\nProtokoll fuer Client " + nr
          + " konnte nicht gestartet werden\n  " + e);
    }
  }

/**
 * Interaktion,
 * liest und verarbeitet Nachrichten vom Client.
 */
  public void run()
  {
    System.out.println
      ( "Protokoll fuer Client " + nr + " gestartet");

    try
    {
      String nachricht;
      do
      {
        nachricht = vomClient.readLine();

        if( nachricht.equalsIgnoreCase( "quit"))
        {
          model.setNachricht
          ( "\tClient " + nr +
            " verlaesst den Chatroom," +
            " Personen im Chatroom: " +
            ( model.countObservers() - 1));

          release();
          return;
        }
        else
        {
          System.out.println
          ( "Client " + nr + " schreibt: " + nachricht);
          model.setNachricht
          ( "\tClient " + nr + " schreibt: " + nachricht);
        }
      } while( true);
    }
    catch( Exception e)
    {
      System.out.println
      ( "\nVerbindung zum Client unterbrochen\n  " + e);
      release();
    }
  }

/**
 * Schliesst Verbindung zum Client,
 * setzt Model zurueck.
 */
  private void release()
  {
    System.out.println
      ( "Protokoll fuer Client " + nr + " beendet");
    try
    {
      s.close();
      model.deleteObserver( this);
      model = null;
    }
    catch( Exception e)
    {
      System.out.println
      ( "\nFehler beim Schliessen des Clientprotokoll\n  "
        + e);
    }
  }

/**
 * Ueberschreibt Interfacemethode update des Observer,
 * teilt Client Aenderung im Modell mit.
 * @param m Modell, welches Aenderungen meldet
 * @param o geaendertes Objekt
 */
  public void update( Observable m, Object o)
  {
    if( model != m) return;

    System.out.println( "Datentransfer Client " + nr);
    zumClient.println( o);
  }
}