// MilchLaden.java                              MM 2007

import Tools.Euklid.*;                           // ggT
import Tools.IO.*;                          // Eingaben

/**
 * Bestimmen der Handlungsschritte beim Abfuellen
 * von 'Soll' Liter mittels einem leeren Eimer 'E'
 * und einer leeren Kanne 'K' mit festgelegtem
 * Fassungsvermoegen.
 */
public class MilchLaden
{
/* ------------------------------------------------- */
// Attribute
/**
 * Behaelter des Kunden.
 */
  private Behaelter kanne;

/**
 * Behaelter des Verkaeufers.
 */
  private Behaelter eimer;

/**
 * Abfuellmenge.
 */
  private int soll;

/**
 * Protokolliert die Aktionen im Milchladen.
 */
  private Protokoll protokoll;

/* ------------------------------------------------- */
// Konstruktor
/**
 * Konstruktor, erzeugt Milchladen mit Protokoll.
 * @param e Behaelter des Verkaeufers
 * @param k Behaelter des Kunden
 * @param s Abfuellmenge
 */
  public MilchLaden( Behaelter e, Behaelter k, int s)
  {
    eimer = e; kanne = k; soll = s;
    protokoll = new Protokoll();
  }

/* ------------------------------------------------- */
// Service-Methoden
/**
 * Behandelt Problem mittels Fallunterscheidung.
 * @return 0 Problem loesbar, 1 sonst.
 */
  public int kannenProblem()
    throws NPlusException
  {
    if( soll == 0)                      // Sonderfaelle
    {
      fertig(); return 0;
    }

    if( soll == kanne.getMax())
    {
      protokoll.nachricht( this.toString());
      kanne.fuellen();
      protokoll.nachricht("\tfuelle K;\n");
      fertig(); return 0;
    }
                               // Nicht loesbare Faelle
    if( soll > kanne.getMax())
    {
      protokoll.nachricht
      ( this.toString() + "\tMilchkanne zu klein!");
      return 1;
    }

    if( soll % 
      Euklid.ggT( eimer.getMax(), kanne.getMax()) != 0)
    {
      protokoll.nachricht
      ( this.toString() + "\tKeine Loesung moeglich!");
      return 1;
    }

    abmessen(); fertig();                 // Normalfall
    return 0;
  }

/**
 * Loest Problem durch Umschuetten.
 */
  private void abmessen()
  {
    while( !kanne.istInhSoll( soll))         // fertig?
    {
      protokoll.nachricht( this.toString());
      eimer.fuellen();
      protokoll.nachricht( "\tfuelle E;\n");

      while( !eimer.istLeer())           // Eimer leer?
      {
        protokoll.nachricht( this.toString());
        kanne.eingiessenAus(eimer);
        protokoll.nachricht( "\tschuette von E nach K;\n");

        if( kanne.istVoll())             // Kanne voll?
        {
          protokoll.nachricht( this.toString());
          kanne.leeren();
          protokoll.nachricht( "\tleere K;\n");
        }
      }
    }
  }

/**
 * Nachricht, dass Problem geloest wurde.
 */
  private void fertig()
  {
    protokoll.nachricht( this.toString() + "\t"
      + kanne.getInh() + " Liter abgefuellt, fertig.");
  }

/* ------------------------------------------------- */
// Zugriffsmethode
/**
 * Stellt Stand in Eimer und Kanne dar.
 */
  public String toString()
  {
    return "E: " + eimer.toString()
       + "\tK: " + kanne.toString();
  }

/* ------------------------------------------------- */
// Startmethode
/**
 * Startet Problem.
 */
  public static void main( String[] args)
  {
    int soll = IOTools.readInteger( "Soll = ");
    int eMax = IOTools.readInteger( "Emax = ");
    int kMax = IOTools.readInteger( "Kmax = ");

    System.out.println( "\nSoll = " + soll + " Emax = "
      + eMax + " Kmax = " + kMax + "\tMilchladen:");

    Behaelter e = new Behaelter( eMax);
    Behaelter k = new Behaelter( kMax);

    MilchLaden laden = new MilchLaden( e, k, soll);
    try
    {
      laden.kannenProblem();
    }
    catch( Exception ex){}
  }
}
