// ObjektDatei.java                                 MM 2012
import java.io.*;              // Inputstream, OutputStream
import java.util.*;                   // Collection, Vector

/**
 * Verwalten von Objektdateien.
 * Lesen aus Datei:         Datei      -->  Collection,
 * Schreiben in Datei:      Collection -->  Datei,
 * Anhaengen an Datei:      Objekt     -->  Datei,
 * Kopieren von/nach Datei: Datei      -->  Datei,
 * Ausgabe aus Collection:  Collection -->  Konsole
 */
public class ObjektDatei<E>
{
/**
 * Lesen aus einer Objektdatei.
 * @param datei Quelldatei
 * @param col Ziel, Collection von Objekten
 */
  public void leseAusObjektDatei
    ( File datei, Collection<E> col)
    throws Exception
  {
    ObjectInputStream inDatei = null;

    try
    {
      inDatei =
      new ObjectInputStream( new FileInputStream( datei));

      while( true) col.add( (E)inDatei.readObject());
    }
                // Dateiende erreicht, Schliessen der Datei
    catch ( EOFException e){}

    catch( Exception e)       // Fehlerbehandlung allgemein
    {
      System.err.println( "Fehler beim Schreiben!");
      throw e;
    }

    finally                             // Aufraeumarbeiten
    {
      try
      {                       // Schliessen der Objektdatei
        if( inDatei != null) inDatei.close();
      }
      catch( IOException e)
      {
        System.err.println
        ( "Fehler beim Schlieen der Objektdatei!" );
      }
    }
  }

/**
 * Schreiben in eine Objektdatei.
 * @param datei Zieldatei
 * @param col Quelle, Collection von Objekten
 */
  public void schreibeInObjektDatei
    ( File datei, Collection<E> col)
    throws Exception
  {
    ObjectOutputStream outDatei = null;

    try
    {
      outDatei =
      new ObjectOutputStream( new FileOutputStream( datei));

      Iterator<E> it = col.iterator();
      while( it.hasNext()) outDatei.writeObject( it.next());
    }

    catch( Exception e)       // Fehlerbehandlung allgemein
    {
      System.err.println( "Fehler beim Schreiben!");
      throw e;
    }

    finally                             // Aufraeumarbeiten
    {
      try
      {                        // Schliessen der Dateidatei
        if( outDatei != null) outDatei.close();
      }
      catch( IOException e)
      {
        System.err.println
        ( "Fehler beim Schlieen der Datendatei!");
      }
    }
  }

/**
 * Schreiben in Objektdatei, anhaengend.
 * @param datei Zieldatei
 * @param obj Quelle, Objekt
 */
  public void schreibeInObjektDatei
    ( File datei, E obj)
    throws Exception
  {
    Collection<E> col = new Vector<E>();
    if( datei.exists())
      leseAusObjektDatei( datei, col);
    col.add( obj);
    schreibeInObjektDatei( datei, col);
  }

/**
 * Kopiert eine Objektdatei objektweise
 * in eine Objektdatei.
 * @param datei0 Quelldatei
 * @param datei1 Zieldatei
 */
  public void kopiereObjektDatei
    ( File datei0, File datei1)
    throws Exception
  {
    Collection<E> col = new Vector<E>();

    leseAusObjektDatei( datei0, col);
    schreibeInObjektDatei( datei1, col);
  }

/**
 * Ausgabe einer Objektliste.
 * @param col Collection von Objekten
 */
  public void listeObjekte( Collection<E> col)
  {
    Iterator<E> it = col.iterator(); int i = 0;
    while( it.hasNext())
      System.out.println( "" + i++ + " " + it.next());
  }
}
