// SpiegelZahl.java                             MM 2007
// Model                                        AZ 2005

import Tools.IO.*;                              // Test

/**
 * Problembehandlung:
 * Erzeugen einer symmetrische Zahl durch Spiegelung.
 */
public class SpiegelZahl
{
/* ------------------------------------------------- */
                                            // coreData
/**
 * Spiegelzahl.
 */
  private Zahl zahl;

/**
 * Schrittanzahl, Anzahl der Spiegelungen.
 */
  private int schritte;

/* ------------------------------------------------- */
                                         // Konstruktor
/**
 * Konstruktor, setzt Startwerte.
 */
  public SpiegelZahl()
  {
    reset();
  }

/* ------------------------------------------------- */
                                    // getData-Methoden
/**
 * Lesen der Zahl.
 * @return Zahl als String
 */
  public String getZahl()
  {
    if( zahl == null) return "";
    return zahl.toString();
  }

/**
 * Lesen der Schrittanzahl.
 * @return Anzahl der ausgefuehrten Spiegelungen
 */
  public int getSchritte()
  {
    return schritte;
  }

/* ------------------------------------------------- */
                                    // service-Methoden
/**
 * Setzen einer Zahl,
 * wirft Ausnahme, falls Sting keine Zahl.
 * @param str Zahl als String
 */
  public void setZahl( String str)
    throws Exception
  {
    zahl = new Zahl( str);
    schritte = 0;
  }

/**
 * Zuruecksetzen einer Zahl.
 */
  public void reset()
  {
    zahl = new Zahl();
    schritte = 0;
  }

/**
 * n mal Spiegeln der Zahl,
 * Abbruch, falls Zahl symmetrisch.
 * @return true, falls Zahl symmetrisch
 */
  public boolean spiegeln( int n)
  {
    if( zahl == null) return true;
    if( zahl.istSymmetrisch()) return true;
    for( ; n > 0; n--)
    {
      schritte ++;
      zahl = zahl.addSpiegel();

      if( zahl.istSymmetrisch()) return true;
    }
    return false;
  }

/* ------------------------------------------------- */
/**
 * Darstellung der Spiegelstandes.
 * @return Schrittanzahl und Wert der Zahl als String
 */
  public String toString()
  {
    return "Schritt: " + schritte +
           "\nZahl: " + zahl.toString();
  }

/* ------------------------------------------------- */
/**
 * Test der Klasse SpiegelZahl.
 */
  public static void main( String[] args)
  {
    SpiegelZahl problem = new SpiegelZahl();

// Eingabe einer Zahl
    while( true)
    {
      try
      {
        String eingabe = IOTools.readLine
          ( "Natuerliche Zahl eingeben: ");
        problem.setZahl( eingabe);
        break;
      }
      catch( Exception e)
      {
        System.err.println( "Eingabefehler!\n" + e);
      }
    }

// Eingabe der Schrittweite
    int schrittweite;
    do
    {
      schrittweite =
        IOTools.readInteger( "Schrittweite eingeben: ");
    } while( schrittweite < 1);

// Spiegeln
    String weiter;
    while( !problem.spiegeln( schrittweite))
    {
      System.out.print( problem);
      weiter = IOTools.readLine( " Weiter j/n: ");
      if( weiter.equals( "n"))
      {
        System.out.println( "\nProblem nicht geloest!\n");
        break;
      }
    }

// Ergebnisausgabe
    System.out.println( problem);
  }
}

/* ------------------------------------------------- */

// Testwerte:
// 789             -> Schrittzahl  4 :          66066
// 53978           -> Schrittzahl  7 :       88255288
// 52007615407     -> Schrittzahl  8 : 89887611678898
// 8396501         -> Schrittzahl 17 : 44440011004444
// 456789900345345 -> nach 46000 Schritten
//                    kein Ergebnis!

