// SpiegelZahlController.java                   MM 2007
// MVC-Controller                               AZ 2005

import javax.swing.*;                    // JOptionPane
import java.awt.event.*; // WindowAdapter, Listener,...

/**
 * Controller zum SpiegelzahlView.
 */
public class SpiegelZahlController
  extends WindowAdapter
  implements ActionListener, KeyListener
{
/**
 * Mathematisches Model,
 * enthaelt Funktionalitaet der Speigelung der Zahlen.
 */
  private SpiegelZahlModel model;

/**
 * Zum Controller gehoeriger View.
 */
  private SpiegelZahlView view;

/* ------------------------------------------------- */
                                    // MVC-Installation
/**
 * Konstruktor, initialisiert Model und View.
 * @param model Mathematisches Modell
 * @param view zum Controller gehoeriger View
 */
  public SpiegelZahlController
  ( SpiegelZahlModel model, SpiegelZahlView view)
  {
    this.model = model;
    this.view = view;
  }

/* ------------------------------------------------- */
                                // handleEvent-Methoden
/**
 * ActionListener,
 * Ereignisauswertung fuer Buttons.
 */
  public void actionPerformed( ActionEvent ae)
  {
    Object command = ae.getActionCommand();
                                  // setzt Startzustand
    if( command.equals( view.ACTION_NEU))
    {
      model.reset();
      view.reset();
    }
                                  // startet Berechnung
    if( command.equals( view.ACTION_START))
    {
      String zahl = view.getTfZahl();
      if( zahl != "")
      {
        try
        {
          model.setZahl( zahl);          // Zahl setzen
          int weite = view.getCbWeite(); // Weite lesen

          if( model.spiegeln( weite))        //Spiegeln
            view.setLbFertig();
          else
            view.setBtWeiter();
        }
        catch( Exception e)
        {
         setFehlerDialog( "Nur Zahlen eingeben!");
         view.reset();             // setzt View zurueck
        }
      }
    }
                            // fuehrt Berechnung weiter
    if( command.equals( view.ACTION_WEITER))
    {
      int weite = view.getCbWeite();     // Weite lesen
      if( model.spiegeln( weite))            //Spiegeln
        view.setLbFertig();
    }
                                  // setzt Schrittweite
    if( command.equals( view.ACTION_SW_JA))
    {
      view.setCbWeite( true);
    }
                          // setzt Schrittweite zurueck
    if( command.equals( view.ACTION_SW_NEIN))
    {
      view.setCbWeite( false);
    }
                                    // Programm beenden
    if( command.equals( view.ACTION_ENDE))
    {
      view.release();
    }
  }

/**
 * Abfangen von Eingabefehlern.
 * @param text Fehlermeldung
 */
  private void setFehlerDialog( String text)
  {
    JOptionPane.showMessageDialog( view, text,
    "Fehlermeldung", JOptionPane.ERROR_MESSAGE);
  }

/* ------------------------------------------------- */

/**
 * KeyListener,
 * Ereignisauswertung Taste wurde gedrueckt,
 * Methode tut nichts.
 */
  public void keyPressed( KeyEvent ke)
  {
  }

/**
 * KeyListener,
 * Ereignisauswertung Taste wurde losgelassen.
 */
  public void keyReleased( KeyEvent ke)
  {
                          // schliesst Zahleneingabe ab
    if( view.istTfZahl( ke.getSource())
        && ke.getKeyChar() == '\n') view.setBtStart();
  }

/**
 * KeyListener,
 * Ereignisauswertung Taste wurde gedrueckt.
 * und losgelassen, Methode tut nichts.
 * 
 */
  public void keyTyped( KeyEvent ke)
  {
  }

/* ------------------------------------------------- */

/**
 * WindowAdapter,
 * Schliessen des Models.
 */
  public void windowClosing( WindowEvent we)
  {
      view.release();
  }

/* ------------------------------------------------- */
                                  // MVC-Deinstallation
/**
 * Freigabe der Controllers,
 * setzt Model und View zurck.
 */
  public void release()
  {
    model = null;
    view = null;
  }
}