// SpiegelZahlModel.java                        MM 2007
// MCV-Model                                    AZ 2005

import Tools.IO.*;                              // Test
import java.util.*;                       // Observable

/**
 * Problembehandlung:
 * Erzeugen einer symmetrische Zahl durch Spiegelung.
 */
public class SpiegelZahlModel extends Observable
{
/* ------------------------------------------------- */
                                            // coreData
/**
 * Spiegelzahl.
 */
  private Zahl zahl;

/**
 * Schrittanzahl, Anzahl der Spiegelungen.
 */
  private int schritte;

/* ------------------------------------------------- */
                                         // Konstruktor
/**
 * Setzt Startwerte.
 */
  public SpiegelZahlModel()
  {
    reset();
  }

/* ------------------------------------------------- */
                                    // getData-Methoden
/**
 * Lesen der Zahl.
 * @return Zahl als String
 */
  public String getZahl()
  {
    return zahl.toString();
  }

/**
 * Lesen der Schrittanzahl.
 * @return Anzahl der ausgefuehrten Spiegelungen
 */
  public int getSchritte()
  {
    return schritte;
  }

/* --------------------------------------------------*/
                                    // service-Methoden
/**
 * Setzen einer Zahl,
 * wirft Ausnahme, falls Sting keine Zahl.
 * @param str Zahl als String
 */
  public void setZahl( String str)
    throws Exception
  {
    zahl = new Zahl( str);
    schritte = 0;

    setChanged();                  // Aenderungsmeldung
    notifyObservers();
  }

/**
 * Setzt Startwerte.
 */
  public void reset()
  {
    zahl = new Zahl();
    schritte = 0;

    setChanged();                  // Aenderungsmeldung
    notifyObservers();
  }

/**
 * n mal Spiegeln der Zahl,
 * Abbruch, falls Zahl symmetrisch.
 * @return true, falls Zahl symmetrisch
 */
  public boolean spiegeln( int n)
  {
    if( zahl == null) return true;
    if( zahl.istSymmetrisch()) return true;
    for( ; n > 0; n--)
    {
      schritte ++;
      zahl = zahl.addSpiegel();

      setChanged();                // Aenderungsmeldung
      notifyObservers();

      if( zahl.istSymmetrisch()) return true;
    }
    return false;
  }

/* ------------------------------------------------- */
/**
 * Darstellung des Spiegelstandes.
 * @return Schrittanzahl und Wert der Zahl als String
 */
  public String toString()
  {
    return "Schritt: " + schritte +
           "\nZahl: " + zahl.toString();
  }
}