// SpiegelZahlView.java                         MM 2007
// MVC-View                                     AZ 2005

import java.awt.*;                             // Color
import javax.swing.*;                // GUI-Komponenten
import java.util.*;                         // Observer

/**
 * View zum SpiegelzahlModel.
 */
public class SpiegelZahlView
  extends JFrame
  implements Observer
{
/* ------------------------------------------------- */
                                      // GUI-Konstanten
/**
 * Konstante, Berechnung wiederholen.
 */
  public final String ACTION_NEU = "Neu";

/**
 * Konstante, Berechnung starten.
 */
  public final String ACTION_START = "Start ";

/**
 * Konstante, Berechnung weiterfuehren.
 */
  public final String ACTION_WEITER = "Weiter";

/**
 * Konstante, Berechnung abgeschlossen.
 */
  public final String ACTION_FERTIG = "Fertig!";

/**
 * Konstante, Programm beenden.
 */
  public final String ACTION_ENDE = "Beenden";

/**
 * Konstante, Schrittweite setzen.
 */
  public final String ACTION_SW_JA = "ja";

/**
 * Konstante, Schrittweite nicht setzen.
 */
  public final String ACTION_SW_NEIN = "nein";

/* ------------------------------------------------- */
                                     // GUI-Komponenten
/**
 * Button Start, Weiter, Fertig,
 * mit wechselnder Aufschrift.
 */
  private JButton btStart;

/**
 * Button Neu.
 */
  private JButton btNeu;

/**
 * Button Beenden.
 */
  private JButton btBeenden;

/**
 * Label Schrittweite.
 */
  private JLabel lbWeite;

/**
 * Label Schrittanzahl.
 */
  private JLabel lbNr;

/**
 * Label Fertig.
 */
  private JLabel lbFertig;

/**
 * Radiobutton Schrittweite Ja/Nein.
 */
  private JRadioButton rbNein, rbJa;

/**
 * Textfeld zur Eingabe der Zahl.
 */
  private JTextField tfZahl;

/**
 * Combobox zur Eingabe der Schrittweite.
 */
  private JComboBox cbWeite;

/**
 * Textfeld zur Ausgabe der Loesung.
 */
  private JTextArea taSpiegel;

/* ------------------------------------------------- */
                                                 // MVC
/**
 * Mathematisches Model,
 * enthaelt Funktionalitaet der Zahlenspiegelung.
 */
  private SpiegelZahlModel model;

/**
 * Zum View gehoeriger Cotroller.
 */
  private SpiegelZahlController controller;

/* ------------------------------------------------- */
                                    // MVC-Installation
/**
 * Konstruktor, setzt Ueberschrift und installiert MVC.
 * @param model Model, welches dargestellt werden soll
 * @param titel Ueberschrift
 */
  public SpiegelZahlView
  ( SpiegelZahlModel model, String titel)
  {
// Titelzeile
    super( titel);
// Model
    this.model = model;
    this.model.addObserver( this);
// Controller
    controller = makeController();
// View
    makeView();
  }

/**
 * Erzeugt Controller, Empfaenger fuer Ereignisse.
 * @return Controller fuer View
 */
  private SpiegelZahlController makeController()
  {
    return new SpiegelZahlController( model, this);
  }

/**
 * Erzeugt View,
 * baut die grafische Oberflaeche auf,
 * Controller verarbeitet Fensterereignisse.
 */
  private void makeView()
  {
// ContentPane
    getContentPane().add( createSpiegelZahlBox());

// Startzustand
    reset();

// Fenster
    addWindowListener( controller);
    pack();
    setVisible( true);
  }

/* ------------------------------------------------- */
/* ------------------------------------------------- */
                                         // ContentPane
/**
 * Senkrechte Box fuer Ein-, Ausgaben und Fertigstatus.
 */
  private Box createSpiegelZahlBox()
  {                                      // Fertiglabel
    lbFertig = new JLabel( ACTION_FERTIG);
    lbFertig.setForeground( Color.RED);

    Box box = Box.createVerticalBox();

    box.add( Box.createVerticalStrut(20));
    box.add( createBoxInOut());    // Ein- und Ausgaben
    box.add( Box.createVerticalStrut(20));
    box.add( lbFertig);                // Fertigmeldung
    box.add( Box.createVerticalStrut(20));

    return box;
  }

/**
 * Waagerechte Box fuer Ein- und Ausgaben.
 */
  private Box createBoxInOut()
  {
    Box box = Box.createHorizontalBox();

    box.add( Box.createHorizontalStrut(20));
    box.add( createBoxIn());              // Eingabebox
    box.add( Box.createHorizontalStrut(30));
    box.add( createBoxOut());             // Ausgabebox
    box.add( Box.createHorizontalStrut(20));

    return box;
  }

/**
 * Senkrechte Box fuer Eingaben.
 */
  private Box createBoxIn()
  {
    Box box = Box.createVerticalBox();

    box.add( createBoxZahl());          // Eingabe Zahl
    box.add( Box.createVerticalStrut(20));
    box.add( createBoxRb());      // Zwischenergebnisse
    box.add( Box.createVerticalStrut(20));
    box.add( createBoxWeite());         // Schrittweite
    box.add( Box.createVerticalStrut(20));
    box.add( createBoxBt());                  // Button

    return box;
  }

/**
 * Waagerechte Box fuer Zahleneingabe,
 * Controller verarbeitet Eingaben.
 */
  private Box createBoxZahl()
  {                                      // Eingabefeld
    tfZahl = new JTextField( 10);
    tfZahl.setHorizontalAlignment( JTextField.RIGHT);
    tfZahl.addKeyListener( controller);

    Box box = Box.createHorizontalBox();

    box.add( new JLabel( "Zahl eingeben:"));
    box.add( Box.createHorizontalStrut(20));
    box.add( tfZahl);

    return box;
  }

/**
 * Waagerechte Box mit zwei Radiobutton,
 * Controller verarbeitet Radiobuttonereignisse.
 */
  private Box createBoxRb()
  {                                      // Radiobutton
    rbNein = new JRadioButton( ACTION_SW_NEIN);
    rbNein.addActionListener( controller);

    rbJa = new JRadioButton( ACTION_SW_JA);
    rbJa.addActionListener( controller);

                         // Gruppierung der Radiobutton
    ButtonGroup bgZwErg = new ButtonGroup();
    bgZwErg.add( rbNein);
    bgZwErg.add( rbJa);

    Box box = Box.createHorizontalBox();

    box.add( new JLabel( "Zwischenergebnisse:"));
    box.add( Box.createHorizontalStrut(20));
    box.add( rbNein);
    box.add( rbJa);

    return box;
  }

/**
 * Waagerechte Box fuer Schrittweite.
 */
  private Box createBoxWeite()
  {
                                      // Label Combobox
    lbWeite = new JLabel( "Schrittweite eingeben:");
                                            // Combobox
    String[] daten = { "1", "2", "5", "10", "20", "50",
                       "100", "200", "500", "1000"};
    cbWeite = new JComboBox( daten);
    cbWeite.setAlignmentX( LEFT_ALIGNMENT);

    Box box = Box.createHorizontalBox();

    box.add( lbWeite);
    box.add( Box.createHorizontalStrut(20));
    box.add( cbWeite);

    return box;
  }

/**
 * Waagerechte Box fuer Steuerbutton,
 * Controller verarbeitet Buttonereignisse.
 */
  private Box createBoxBt()
  {
    btStart   = new JButton( ACTION_START);
    btStart.addActionListener( controller);

    btNeu     = new JButton( ACTION_NEU);
    btNeu.addActionListener( controller);

    btBeenden = new JButton( ACTION_ENDE);
    btBeenden.addActionListener( controller);

    Box box = Box.createHorizontalBox();

    box.add( btStart);
    box.add( btNeu);
    box.add( btBeenden);

    return box;
  }

/**
 * Senkrechte Box fuer Ausgaben.
 */
  private Box createBoxOut()
  {
    lbNr  = new JLabel( "0");               // Schritte

                                // Ausgabe mit Umbruch,
              // nicht editierbar und nicht focusierbar
    taSpiegel = new JTextArea( 3, 20);
    taSpiegel.setLineWrap( true);
    taSpiegel.setEditable( false);
    taSpiegel.setFocusable( false);

                          // Rollbalken und Ausrichtung
    JScrollPane spSpiegel = 
      new JScrollPane( taSpiegel,
          JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
          JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    spSpiegel.setAlignmentX(0);

    Box box = Box.createVerticalBox();

    box.add( new JLabel( "Schritt:"));                   // Schritt
    box.add( Box.createVerticalStrut(20));
    box.add( lbNr);
    box.add( Box.createVerticalStrut(20));
    box.add( new JLabel( "Ergebnis:"));                 // Ergebnis
    box.add( Box.createVerticalStrut(20));
    box.add( spSpiegel);                     // Ausgabe

    return box;
  }

/* ------------------------------------------------- */
/* ------------------------------------------------- */
                    // service-Methoden fuer Controller
/**
 * Setzt Startzustand, Button Neu.
 */
  public void reset()
  {
// Eingabefeld
    tfZahl.setText( "");
    tfZahl.setEditable( true);
    tfZahl.requestFocus();

// Radiobutton und Schrittweite
    rbNein.setSelected( true);
    lbWeite.setEnabled( false);
    cbWeite.setEnabled( false);
    cbWeite.setSelectedIndex( 0);

// Button
    btStart.setText( ACTION_START);
    btStart.setEnabled( false);

// Ausgabefeld
    lbNr.setText( "0");
    taSpiegel.setText( "");

// Fertiglabel
    lbFertig.setEnabled( false);
  }

/**
 * Button Start.
 */
  public void setBtStart()
  {
    tfZahl.setEditable( false);
    btStart.setEnabled( true);
    btStart.requestFocus();
  }

/**
 * Button Weiter.
 */
  public void setBtWeiter()
  {
    btStart.setText( ACTION_WEITER);
    btStart.requestFocus();
  }

/**
 * Fertig, Zahl symmetrisch.
 */
  public void setLbFertig()
  {
    tfZahl.setEditable( false);
    btStart.setEnabled( false);
    btStart.setText( ACTION_FERTIG);
    lbFertig.setEnabled( true);

    btNeu.requestFocus();
  }

/**
 * Setzen/Nichtsetzen der Schrittweite.
 */
  public void setCbWeite( boolean b)
  {
    lbWeite.setEnabled( b);
    cbWeite.setEnabled( b);
    cbWeite.setSelectedIndex( 0);

    setBtStart();
  }

/**
 * Lesen der Schrittweite aus ComboBox.
 * @return gewuenschte Schrittweite
 */
  public int getCbWeite()
  {
    return Integer.
          parseInt( (String)cbWeite.getSelectedItem());
  }

/**
 * Ereignisquelle Textfeld.
 * @return true, falls Zahl eingegeben wurde
 */
  public boolean istTfZahl( Object quelle)
  {
    return quelle == tfZahl;
  }

/**
 * Lesen Inhalt des Textfelds.
 * @return Zahl als String
 */
  public String getTfZahl()
  {
    return tfZahl.getText();
  }

/* ------------------------------------------------- */
/* ------------------------------------------------- */
                                    // Observer-Methode
/**
 * Ueberschreiben der Interfacemethode update,
 * legt Reaktion auf Aenderungen fest.
 * @param m Model, welches Aenderungen meldet
 * @param o geanderte Objekte
 */
  public void update( Observable m, Object o)
  {
    if( m == model)
    {
      taSpiegel.setText( model.getZahl());
      lbNr.setText( "" + model.getSchritte());
    }
  }

/* ------------------------------------------------- */
                                  // MVC-Deinstallation
/**
 * Setzt Model und Controller zurck.
 */
  public void release()
  {
// Fenster
    dispose();
// Controller
    controller.release();
    controller = null;
// Model
    model.deleteObserver( this);
    model = null;
  }
}