// TextDatei.java                                   MM 2012
import java.io.*;                         // Reader, Writer

/**
 * Gepuffertes Kopieren von Textdateien.
 * Aufruf: java TextDatei <Quelldatei> <Zieldatei>
 * Schreiben in Datei:        Tastatur  -->  Datei
 * Kopieren von/nach Datei:   Datei     -->  Datei
 * Lesen aus Datei:           Datei     -->  Konsole
 */
public class TextDatei
{
/**
 * Kopiert zeilenweise, gepuffert,
 * schliesst nach Beenden die Dateien.
 * @param inBuffer Quelle
 * @param outBuffer Ziel
 */
  private void kopieren
    ( BufferedReader inBuffer, BufferedWriter outBuffer)
    throws Exception
  {
    try
    {
      while( true)
      {                              // Lesen und Schreiben
        outBuffer.write( inBuffer.readLine());
        outBuffer.newLine();               // Zeilenwechsel
      }
    }

    catch( NullPointerException e){}  // Dateiende erreicht
  }

/**
 * Liest Text von der Tastatur
 * und schreibt ihn in eine Datei.
 * @param datei Zieldatei
 */
  public void schreibeInTextDatei( File datei)
    throws Exception
  {
    BufferedReader inBuffer = null;
    BufferedWriter outBuffer = null;

    try
    {               // Tastatureingabedatenstrom, gepuffert
      inBuffer =
      new BufferedReader( new InputStreamReader( System.in));
                  // Ausgabedatenstrom auf datei, gepuffert
      outBuffer =
      new BufferedWriter( new FileWriter( datei));

      kopieren( inBuffer, outBuffer);           // Kopieren
    }

    catch( Exception e)       // Fehlerbehandlung allgemein
    {
      System.err.println( "Fehler beim Schreiben!");
      throw e;
    }

    finally                             // Aufraeumarbeiten
    {
      try
      {                             // Schliessen der Datei
        if( outBuffer != null) outBuffer.close();
      }
      catch( IOException e)
      {
        System.err.println
        ( "Fehler beim Schlieen der Textdatei!" );
      }
    }
  }

/**
 * Kopiert eine Textdatei zeilenweise in eine Textdatei.
 * @param datei0 Quelldatei
 * @param datei1 Zieldatei
 */
  public void kopiereTextDatei( File datei0, File datei1)
    throws Exception
  {
    BufferedReader inBuffer = null;
    BufferedWriter outBuffer = null;

    try
    {            // Eingabedatenstrom von datei0, gepuffert
      inBuffer =
      new BufferedReader( new FileReader( datei0));
                 // Ausgabedatenstrom auf datei1, gepuffert
      outBuffer =
      new BufferedWriter( new FileWriter( datei1));

      kopieren( inBuffer, outBuffer);           // Kopieren
    }

    catch( Exception e)       // Fehlerbehandlung allgemein
    {
      System.err.println( "Fehler beim Schreiben!");
      throw e;
    }

    finally                             // Aufraeumarbeiten
    {
      try
      {                           // Schliessen der Dateien
        if( inBuffer != null) inBuffer.close();
        if( outBuffer != null) outBuffer.close();
      }
      catch( IOException e)
      {
        System.err.println
        ( "Fehler beim Schlieen der Textdateien!" );
      }
    }
  }

/**
 * Gibt Datei zeilenweise auf der Konsole aus.
 * @param datei Quelldatei
 */
  public void leseAusTextDatei( File datei)
    throws Exception
  {
    BufferedReader inBuffer = null;
    BufferedWriter outBuffer = null;

    try
    {             // Eingabedatenstrom von datei, gepuffert
      inBuffer =
      new BufferedReader( new FileReader( datei));
                // Ausgabedatenstrom auf Konsole, gepuffert
      outBuffer =
      new BufferedWriter(new OutputStreamWriter(System.out));

                                       // Ausgabe der Datei
      kopieren( inBuffer, outBuffer);
    }

    catch( Exception e)       // Fehlerbehandlung allgemein
    {
      System.err.println( "Fehler beim Schreiben!");
      throw e;
    }

    finally                             // Aufraeumarbeiten
    {
      try
      {                         // Schliessen der Textdatei
        if( inBuffer != null) inBuffer.close();
                               // Leeren des Ausgabepuffers
        if( outBuffer != null) outBuffer.flush();
      }
      catch( IOException e)
      {
        System.err.println
        ( "Fehler beim Schlieen der Textdatei!" );
      }
    }
  }

/**
 * Testprogramm,
 * Tastatur --> Datei1 --> Datei2 --> Konsole.
 */
  public static void main( String[] args)
  {
    TextDatei text = new TextDatei();

    try
    {
      File datei0 = new File( args[ 0]);
      File datei1 = new File( args[ 1]);

// Schreiben in Datei:   Tastatur     -->  Datei datei0
      System.out.println( "Schreiben des Textes");
      System.out.print( "Text eingeben, ");
      System.out.println
      ( "Abbruch: Strg+Z/Ctrl+Z bzw. Strg+D/Ctrl+D:");
      text.schreibeInTextDatei( datei0);
      System.out.println( "Schreiben abgeschlossen!\n");

// Kopie von/nach datei: Datei datei0 -->  Datei datei1
      System.out.println( "Kopieren der Datei");
      text.kopiereTextDatei( datei0, datei1);
      System.out.println( "Kopieren abgeschlossen!\n");

// Lesen aus Datei:      Datei datei1 -->  Konsole
      System.out.println( "Lesen der Kopie");
      text.leseAusTextDatei( datei1);
      System.out.println( "Lesen abgeschlossen!\n");
    }

                               // Befehl nicht vollstaendig
    catch( ArrayIndexOutOfBoundsException e)
    {
      System.err.println
      ( "Auruf: java TextDatei <Quelldatei> <Zieldatei>");
    }


    catch( Exception e)       // Fehlerbehandlung allgemein
    {
      System.err.println( "Fehler: " + e);
    }

    System.out.println( "Fertig!");
  }
}